@@ -1,6 +1,9 @@
+import sys
 import ntpath
 import os
 import shutil
+import importlib
+import importlib.util
 from copy import deepcopy
 from pathlib import Path
 import yaml
@@ -13,10 +16,11 @@ from steam_sdk.drivers.DriverFiQuS import DriverFiQuS
 from steam_sdk.drivers.DriverLEDET import DriverLEDET
 from steam_sdk.drivers.DriverPSPICE import DriverPSPICE
 from steam_sdk.drivers.DriverPyBBQ import DriverPyBBQ
-from steam_sdk.parsers.ParserYAML import dict_to_yaml
+from steam_sdk.parsers.ParserYAML import dict_to_yaml, yaml_to_data_class
 from steam_sdk.utils.make_folder_if_not_existing import make_folder_if_not_existing
 from steam_sdk.utils.rgetattr import rgetattr
 from steam_sdk.utils.sgetattr import rsetattr
+from steam_sdk.viewers.Viewer import Viewer
 
 
 class AnalysisSTEAM:
@@ -42,15 +46,17 @@ class AnalysisSTEAM:
         self.temp_path     = None
         self.list_models   = {}  # this dictionary will be populated with BuilderModel objects and their names
         self.list_sims     = []  # this list will be populated with integers indicating simulations to run
+        self.list_viewers  = {}  # this dictionary will be populated with Viewer objects and their names
 
         # Check that input file is provided
         if not self.file_name_analysis:
             raise Exception('No .yaml input file provided.')
 
         # Load yaml keys into DataAnalysis dataclass
-        with open(self.file_name_analysis, "r") as stream:
-            dictionary_yaml = yaml.safe_load(stream)
-            self.data_analysis = DataAnalysis(**dictionary_yaml)
+        self.data_analysis = yaml_to_data_class(self.file_name_analysis, DataAnalysis)
+        # with open(self.file_name_analysis, "r") as stream:
+        #     dictionary_yaml = yaml.safe_load(stream)
+        #     self.data_analysis = DataAnalysis(**dictionary_yaml)
 
         # Read working folders and set them up
         self.set_up_working_folders()
@@ -170,12 +176,12 @@ class AnalysisSTEAM:
         if verbose: print('File {} was saved locally.'.format(path_temp_file_settings))
 
     def write_analysis_file(self, path_output_file: str):
-        '''
+        """
         ** Write the analysis data in the target file **
         This can be helpful to keep track of the final state of the DataAnalysis object before running it, especially if it was modified programmatically.
         :param path_output_file: string to the file to write
         :return: None
-        '''
+        """
 
         verbose = self.verbose
 
@@ -206,7 +212,7 @@ class AnalysisSTEAM:
         # Print analysis sequence
         if verbose: print('Defined sequence of analysis steps:')
         for s, seq_step in enumerate(self.data_analysis.AnalysisStepSequence):
-            if verbose: print('Step {}/{}: {}'.format(s+1, len(self.data_analysis.AnalysisStepSequence), seq_step))
+            if verbose: print('Step {}/{}: {}'.format(s + 1, len(self.data_analysis.AnalysisStepSequence), seq_step))
 
         # Run analysis (and re-print analysis steps)
         if verbose: print('Analysis started.')
@@ -230,6 +236,12 @@ class AnalysisSTEAM:
                 self.add_auxiliary_file(step, verbose=verbose)
             elif step.type == 'CopyFile':
                 self.copy_file_to_target(step, verbose=verbose)
+            elif step.type == 'RunCustomPyFunction':
+                self.run_custom_py_function(step, verbose=verbose)
+            elif step.type == 'RunViewer':
+                self.run_viewer(step, verbose=verbose)
+            elif step.type == 'CalculateMetrics':
+                self.calculate_metrics(step, verbose=verbose)
             else:
                 raise Exception('Unknown type of analysis step: {}'.format(step.type))
 
@@ -241,7 +253,7 @@ class AnalysisSTEAM:
         if (str(self.library_path).endswith('steam_models')) or (str(self.library_path).endswith('steam-models')) or (str(self.library_path).endswith('model_library')):
             file_model_data = os.path.join(self.library_path, step.case_model + 's', step.file_model_data, 'input', 'modelData_' + step.file_model_data + '.yaml')
         else:
-            file_model_data = os.path.join(self.library_path,'modelData_' + step.file_model_data + '.yaml')
+            file_model_data = os.path.join(self.library_path, 'modelData_' + step.file_model_data + '.yaml')
 
         case_model      = step.case_model
         software        = step.software  # remember this is a list, not a string
@@ -262,7 +274,8 @@ class AnalysisSTEAM:
         # Build simulation file
         if step.simulation_number:
             if 'FiQuS' in step.software:
-                self.setup_sim_FiQuS(simulation_name=step.simulation_name, simulation_type=step.type, sim_number=step.simulation_number)
+                self.setup_sim_FiQuS(magnet_type=BM.model_data.GeneralParameters.magnet_type, simulation_name=step.simulation_name,
+                                     simulation_type=step.type, sim_number=step.simulation_number)
             if 'LEDET' in step.software:
                 flag_yaml = True  # Hard-coded for the moment
                 self.setup_sim_LEDET(simulation_name=step.simulation_name, sim_number=step.simulation_number, flag_yaml=flag_yaml, flag_json=flag_json)
@@ -306,12 +319,20 @@ class AnalysisSTEAM:
                 if len_new_model_name > 0:  # Make a new copy of the BuilderModel object, and change it
                     self.list_models[step.new_model_name[v]] = deepcopy(BM)
                     BM = self.list_models[step.new_model_name[v]]
-                    rsetattr(BM.model_data.Conductors[idx_conductor], conductor_variable_to_change, value)
+
+                    if case_model == 'conductor':
+                        rsetattr(BM.conductor_data.Conductors[idx_conductor], conductor_variable_to_change, value)
+                    else:
+                        rsetattr(BM.model_data.Conductors[idx_conductor], conductor_variable_to_change, value)
+
                     if verbose:
                         print('Model {} copied to model {}.'.format(step.model_name, step.new_model_name[v]))
-
                 else:  # Change the original BuilderModel object
-                    rsetattr(BM.model_data.Conductors[idx_conductor], conductor_variable_to_change, value)
+
+                    if case_model == 'conductor':
+                        rsetattr(BM.conductor_data.Conductors[idx_conductor], conductor_variable_to_change, value)
+                    else:
+                        rsetattr(BM.model_data.Conductors[idx_conductor], conductor_variable_to_change, value)
 
             else:  # Standard case when the variable to change is not the Conductors key
                 if verbose:
@@ -328,16 +349,21 @@ class AnalysisSTEAM:
                 else:  # Change the original BuilderModel object
                     self.set_attribute_model(case_model, BM, step.variable_to_change, value)
 
+            # Special case: If the sub-keys of "Source" are changed, a resetting of the input paths is triggered
+            if step.variable_to_change.startswith('Sources.'):
+                BM.set_input_paths()
+
             # Build simulation file
             if len_simulation_numbers > 0:
                 simulation_number = step.simulation_numbers[v]
                 if 'FiQuS' in step.software:
                     BM.buildFiQuS()
-                    self.setup_sim_FiQuS(simulation_name=step.simulation_name, simulation_type=step.type, sim_number=simulation_number)
+                    self.setup_sim_FiQuS(magnet_type=BM.model_data.GeneralParameters.magnet_type, simulation_name=step.simulation_name,
+                                         model_name=step.model_name, simulation_type=step.type, sim_number=simulation_number)
                 if 'LEDET' in step.software:
                     flag_json = BM.flag_json
                     flag_yaml = True  # Hard-coded for the moment
-                    BM.buildLEDET(case_model=case_model)
+                    BM.buildLEDET()
                     self.setup_sim_LEDET(simulation_name=step.simulation_name, sim_number=simulation_number, flag_yaml=flag_yaml, flag_json=flag_json)
                 if 'PyBBQ' in step.software:
                     BM.buildPyBBQ()
@@ -560,12 +586,161 @@ class AnalysisSTEAM:
         shutil.copyfile(full_path_file_to_copy, full_path_file_target)
         if verbose: print(f'File {full_path_file_to_copy} was copied to {full_path_file_target}.')
 
-    def setup_sim_FiQuS(self, simulation_name, simulation_type, sim_number):
+
+    def run_custom_py_function(self, step, verbose: bool = False):
+        """
+            Run a custom Python function with given arguments
+        """
+        # If the step is not enabled, the function will not be run
+        if not step.flag_enable:
+            if verbose: print(f'flag_enable set to False. Custom function {step.function_name} will not be run.')
+            return
+
+        # Unpack variables
+        function_name = step.function_name
+        function_arguments = step.function_arguments
+        if step.path_module:
+            # Import the custom function from a specified location different from the default location
+            # This Python magic comes from: https://stackoverflow.com/questions/67631/how-do-i-import-a-module-given-the-full-path
+            path_module = os.path.join(Path(step.path_module).resolve())
+            custom_module = importlib.util.spec_from_file_location('custom_module', os.path.join(path_module, function_name + '.py'))
+            custom_function_to_load = importlib.util.module_from_spec(custom_module)
+            sys.modules['custom_module'] = custom_function_to_load
+            custom_module.loader.exec_module(custom_function_to_load)
+            custom_function = getattr(custom_function_to_load, function_name)
+        else:
+            # Import the custom function from the default location
+            path_module = f'steam_sdk.analyses.custom_analyses.{function_name}.{function_name}'
+            custom_module = importlib.import_module(path_module)
+            custom_function = getattr(custom_module, function_name)
+
+        # Run custom function with the given argument
+        if verbose: print(f'Custom function {function_name} from module {path_module} will be run with arguments: {function_arguments}.')
+        output = custom_function(function_arguments)
+        return output
+
+    def run_viewer(self, step, verbose: bool = False):
+        """
+            Make a steam_sdk.viewers.Viewer.Viewer() object and run its analysis
+        """
+
+        # Unpack variables
+        viewer_name = step.viewer_name
+
+        if verbose: print(f'Making Viewer object named {viewer_name}.')
+
+        # Make a steam_sdk.viewers.Viewer.Viewer() object and run its analysis
+        V = Viewer(file_name_transients=step.file_name_transients,
+                   list_events=step.list_events,
+                   flag_analyze=step.flag_analyze,
+                   flag_display=step.flag_display,
+                   flag_save_figures=step.flag_save_figures,
+                   verbose=step.verbose)
+
+        # Add the reference to the Viewer object in the dictionary
+        self.list_viewers[viewer_name] = V
+
+    def calculate_metrics(self, step, verbose: bool = False):
+        """
+        Calculate metrics (usually to compare two or more measured and/or simulated signals)
+        :param step: STEAM analysis step of type CalculateMetrics, which has attributes:
+        - viewer_name: the name of the Viewer object containing the data to analyze
+        - metrics_to_calculate: list that defines the type of calculation to perform for each metric.
+        - variables_to_analyze: list
+        :param verbose:
+        :return:
+        """
+        """
+            
+            The metrics to calculate are indicated in the list metrics_to_calculate, which defines the type of calculation of each metric.
+
+        """
+        if verbose: print(f'Calculate metrics.')
+
+        # Unpack variables
+        viewer_name = step.viewer_name
+        metrics_to_calculate = step.metrics_to_calculate
+        variables_to_analyze = step.variables_to_analyze
+        # Note: Avoid unpacking "list_viewers = self.list_viewers" since the variable usually has large size
+
+        # Check input
+        if not viewer_name in self.list_viewers:
+            raise Exception(f'The selected Viewer object named {viewer_name} is not present in the current Viewer list: {self.list_viewers}. Add an analysis step of type RunViewer to define a Viewer object.')
+        # if len(metrics_to_calculate) != len(variables_to_analyze):
+        #     raise Exception(f'The lengths of the lists metrics_to_calculate and variables_to_analyze must match, but are {len(metrics_to_calculate)} and {len(variables_to_analyze)} instead.')
+
+        # Loop through all events listed in the selected Viewer object
+        for event_id in self.list_viewers[viewer_name].list_events:
+            event_label        = self.list_viewers[viewer_name].dict_events['Event label'][event_id-1]
+            if verbose: print(f'Event #{event_id}: "{event_label}".')
+
+            # For each selected pair of variables to analyze, calculate metrics
+            for pair_var in variables_to_analyze:
+                var_to_analyze = pair_var[0]
+                var_reference  = pair_var[1]
+
+                # Check that the selected signal to analyze and its reference signal (if they are defined) exist in the current event
+                if len(var_to_analyze) > 0:
+                    if var_to_analyze in self.list_viewers[viewer_name].dict_data[event_label]:
+                        if 'x_sim' in self.list_viewers[viewer_name].dict_data[event_label][var_to_analyze]:  # usually the variable to analyze is a simulated signal
+                            x_var_to_analyze = self.list_viewers[viewer_name].dict_data[event_label][var_to_analyze]['x_sim']
+                            y_var_to_analyze = self.list_viewers[viewer_name].dict_data[event_label][var_to_analyze]['y_sim']
+                        elif 'x_meas' in self.list_viewers[viewer_name].dict_data[event_label][var_to_analyze]:  # but a measured signal is also supported
+                            x_var_to_analyze = self.list_viewers[viewer_name].dict_data[event_label][var_to_analyze]['x_meas']
+                            y_var_to_analyze = self.list_viewers[viewer_name].dict_data[event_label][var_to_analyze]['y_meas']
+                        else:
+                            print(f'WARNING: Viewer {viewer_name}: Event "{event_label}": Signal label "{var_to_analyze}" not found. Signal skipped.')
+                            continue
+                    else:
+                        print(f'WARNING: Viewer "{viewer_name}": Event "{event_label}": Signal label "{var_to_analyze}" not found. Signal skipped.')
+                        continue
+                else:
+                    raise Exception(f'Viewer "{viewer_name}": Event "{event_label}": The first value of each pair in variables_to_analyze cannot be left empty, but {pair_var} was found.')
+
+                if len(var_reference) > 0:  # if the string is empty, skip this check (it is possible to run the metrics calculation on one variable only)
+                    if var_reference in self.list_viewers[viewer_name].dict_data[event_label]:
+                        if 'x_meas' in self.list_viewers[viewer_name].dict_data[event_label][var_reference]:  # usually the variable to analyze is a measured signal
+                            x_var_reference = self.list_viewers[viewer_name].dict_data[event_label][var_reference]['x_meas']
+                            y_var_reference = self.list_viewers[viewer_name].dict_data[event_label][var_reference]['y_meas']
+                        elif 'x_sim' in self.list_viewers[viewer_name].dict_data[event_label][var_reference]:  # but a simulated signal is also supported
+                            x_var_reference = self.list_viewers[viewer_name].dict_data[event_label][var_reference]['x_sim']
+                            y_var_reference = self.list_viewers[viewer_name].dict_data[event_label][var_reference]['y_sim']
+                        else:
+                            print(f'WARNING: Viewer "{viewer_name}": Event "{event_label}": Signal label "{var_reference}" not found. Signal skipped.')
+                            continue
+                    else:
+                        print(f'WARNING: Viewer "{viewer_name}": Event "{event_label}": Signal label "{var_reference}" not found. Signal skipped.')
+                        continue
+                else:  # It is possible to run the metrics calculation on one variable only, without a reference signal
+                    x_var_reference = None
+                    y_var_reference = None
+
+                # Perform the metrics calculation
+                if verbose: print(f'Viewer "{viewer_name}": Event "{event_label}": Metrics calculated using signals "{var_to_analyze}" and "{var_reference}".')
+
+                # Calculate the metrics
+                # output_metric = Metrics(
+                #     metrics_to_calculate=metrics_to_calculate,
+                #     x_value=y_var_to_analyze,
+                #     y_value=y_var_to_analyze,
+                #     x_ref=x_var_reference,
+                #     y_ref=y_var_reference,
+                #     flag_run=True)
+
+
+    def setup_sim_FiQuS(self, simulation_name, simulation_type, sim_number, model_name=None, magnet_type=None):
         """
         Set up a FiQuS simulation by copying the last file generated by BuilderModel to the output folder and to the
         local FiQuS working folder.
         The original file is then deleted.
         """
+        def _find_original_model(name):
+            src = None
+            for k, s in self.data_analysis.AnalysisStepDefinition.items():
+                if s.type == 'MakeModel':
+                    if s.model_name == name:
+                        src = os.path.join(os.path.dirname(local_model_folder), str(s.simulation_number), simulation_name)
+            return src
 
         # Unpack
         output_folder = self.output_path
@@ -575,23 +750,34 @@ class AnalysisSTEAM:
         self.list_sims.append(sim_number)
 
         # Make simulation folder
-        local_model_folder = os.path.join(local_FiQuS_folder, simulation_name, str(sim_number))
+        local_model_folder = os.path.join(local_FiQuS_folder, 'Analysis_' + simulation_name, str(sim_number))
         make_folder_if_not_existing(local_model_folder)
 
         # Copy simulation file
         file_name_temp = os.path.join(output_folder, simulation_name)
-        yaml_temp = os.path.join(file_name_temp + '.yaml')
+        yaml_temp = os.path.join(file_name_temp + '_FiQuS.yaml')
         file_name_local = os.path.join(local_model_folder, simulation_name)
         yaml_local = os.path.join(file_name_local + '.yaml')
         shutil.copyfile(yaml_temp, yaml_local)
 
-        if simulation_type == 'MakeModel':
-            geo_temp = os.path.join(file_name_temp + '.geom')
-            set_temp = os.path.join(file_name_temp + '.set')
-            geo_local = os.path.join(file_name_local + '.geom')
-            set_local = os.path.join(file_name_local + '.set')
-            shutil.copyfile(geo_temp, geo_local)
-            shutil.copyfile(set_temp, set_local)
+        if magnet_type == 'multipole':
+            if simulation_type == 'MakeModel':
+                geo_temp = os.path.join(file_name_temp + '_FiQuS.geom')
+                set_temp = os.path.join(file_name_temp + '_FiQuS.set')
+                geo_local = os.path.join(file_name_local + '.geom')
+                set_local = os.path.join(file_name_local + '.set')
+                shutil.copyfile(geo_temp, geo_local)
+                shutil.copyfile(set_temp, set_local)
+            else:
+                source = _find_original_model(model_name)
+                while not source:
+                    for key, step in self.data_analysis.AnalysisStepDefinition.items():
+                        if step.type in ['ModifyModel', 'ModifyModelMultipleVariables']:
+                            if model_name in step.new_model_name:
+                                new_name = step.model_name
+                                source = _find_original_model(new_name)
+                os.symlink(source + '.geom', file_name_local + '.geom')
+                os.symlink(source + '.set', file_name_local + '.set')
 
         if self.verbose: print('Simulation files {} generated.'.format(file_name_temp))
         if self.verbose: print('Simulation files {} copied.'.format(file_name_local))
@@ -716,11 +902,12 @@ class AnalysisSTEAM:
         The function applies a different logic for each simulation software.
         """
         if software == 'FiQuS':
-            local_model_folder_input = os.path.join(self.settings.local_FiQuS_folder, simulation_name, str(sim_number))
-            relative_folder_output = os.path.join(simulation_name, str(sim_number))
-            dFiQuS = DriverFiQuS(path_exe=self.settings.FiQuS_path, path_folder_FiQuS=self.settings.local_FiQuS_folder,
+            local_analysis_folder = 'Analysis_' + simulation_name
+            local_model_folder_input = os.path.join(self.settings.local_FiQuS_folder, local_analysis_folder, str(sim_number))
+            dFiQuS = DriverFiQuS(FiQuS_path=self.settings.FiQuS_path, path_folder_FiQuS=self.settings.local_FiQuS_folder,
                                  path_folder_FiQuS_input=local_model_folder_input, verbose=verbose)
-            dFiQuS.run_FiQuS(simulation_name, outputDirectory=relative_folder_output)
+            dFiQuS.run_FiQuS(sim_file_name=simulation_name,
+                             output_directory=os.path.join(local_analysis_folder, simulation_name))
         elif software == 'LEDET':
             dLEDET = DriverLEDET(path_exe=self.settings.LEDET_path, path_folder_LEDET=self.settings.local_LEDET_folder, verbose=verbose)
             dLEDET.run_LEDET(simulation_name, str(sim_number), simFileType=simFileType)
@@ -735,3 +922,4 @@ class AnalysisSTEAM:
             dPSPICE.run_PSPICE(simulation_name, suffix='')
         else:
             raise Exception(f'Software {software} not supported for automated running.')
+
