@@ -1,11 +1,10 @@
 import os
 import shutil
 from pathlib import Path
-import yaml
 
 from steam_sdk.builders.BuilderPyBBQ import BuilderPyBBQ
 from steam_sdk.data.DataModelConductor import DataModelConductor
-from steam_sdk.data.DataModelMagnet import *
+from steam_sdk.data.DataModelMagnet import DataModelMagnet
 from steam_sdk.data.DataModelCircuit import DataModelCircuit
 from steam_sdk.data.DataRoxieParser import RoxieData
 from steam_sdk.builders.BuilderFiQuS import BuilderFiQuS
@@ -19,7 +18,7 @@ from steam_sdk.parsers.ParserProteCCT import ParserProteCCT
 from steam_sdk.parsers.ParserProtePyBBQ import ParserPyBBQ
 from steam_sdk.parsers.ParserRoxie import ParserRoxie
 from steam_sdk.parsers.ParserMap2d import getParametersFromMap2d
-from steam_sdk.parsers.ParserYAML import dict_to_yaml
+from steam_sdk.parsers.ParserYAML import dict_to_yaml, yaml_to_dict, yaml_to_data_class
 from steam_sdk.plotters import PlotterRoxie
 from steam_sdk.plotters.PlotterModel import PlotterModel
 
@@ -30,7 +29,7 @@ class BuilderModel:
     """
 
     def __init__(self,
-                 file_model_data: str = None, software: List[str] = None, case_model: str = 'magnet',
+                 file_model_data: str = None, software: [str] = None, case_model: str = 'magnet',
                  relative_path_settings: str = '',
                  flag_build: bool = True, flag_dump_all: bool = False, flag_json: bool = False,
                  dump_all_path: str = '', output_path: str = '',
@@ -43,15 +42,14 @@ class BuilderModel:
         """
 
         # Unpack arguments
-        self.software: List[str] = software
-        if not self.software: self.software = []  # This avoids error when None is passed to software # TODO: Verify this is sound coding
-        valid_cases = ['magnet', 'circuit', 'conductor']
-        if case_model in valid_cases:
-            self.case_model = case_model
-        else:
-            raise ValueError('results: case_model must be one of {}.'.format(valid_cases))
-        self.relative_path_settings: str = relative_path_settings
         self.file_model_data: str = file_model_data
+        if software is None:
+            self.software = []
+        else:
+            self.software = software
+        self.case_model = case_model
+        self.relative_path_settings: str = relative_path_settings
+
         self.flag_build = flag_build
         self.flag_dump_all: bool = flag_dump_all
         self.flag_plot_all: bool = flag_plot_all
@@ -61,7 +59,7 @@ class BuilderModel:
         self.verbose: bool = verbose
 
         if self.verbose:
-            print('case_model: {}'.format(case_model))
+            print('case_model: {}'.format(self.case_model))
 
         # If a model needs to be built, the output folder is not an empty string, and the folder does not exist, make it
         if flag_build and self.output_path != '' and not os.path.isdir(self.output_path):
@@ -69,7 +67,7 @@ class BuilderModel:
             Path(self.output_path).mkdir(parents=True)
 
         ### Case of a magnet model
-        if case_model == 'magnet':
+        if self.case_model == 'magnet':
             # Initialize
             self.model_data: DataModelMagnet = DataModelMagnet()
             self.roxie_data: RoxieData = RoxieData()
@@ -86,10 +84,10 @@ class BuilderModel:
                     print('output_path: {}'.format(self.output_path))
 
                 # Load model data from the input .yaml file
-                self.loadModelData(self.case_model)
+                self.loadModelData()
 
                 # Set paths of input files and settings
-                self.set_input_paths(self.case_model)
+                self.set_input_paths()
 
                 # Load model data from the input ROXIE files
                 self.loadRoxieData()
@@ -97,7 +95,7 @@ class BuilderModel:
                 # Build model
                 for s in self.software:
                     if s == 'FiQuS':    self.buildFiQuS()
-                    if s == 'LEDET':    self.buildLEDET(case_model=self.case_model)
+                    if s == 'LEDET':    self.buildLEDET()
                     if s == 'ProteCCT': self.buildProteCCT()
                     if s == 'SIGMA':    self.buildSIGMA()
 
@@ -110,7 +108,7 @@ class BuilderModel:
                     PM.plot_all(self.model_data)
 
         ### Case of a circuit model
-        elif case_model == 'circuit':
+        elif self.case_model == 'circuit':
             # Initialize
             self.circuit_data: DataModelCircuit = DataModelCircuit()
             self.path_settings = None
@@ -121,17 +119,17 @@ class BuilderModel:
                     print('output_path: {}'.format(self.output_path))
 
                 # Load model data from the input .yaml file
-                self.loadModelData(self.case_model)
+                self.loadModelData()
 
                 # Set paths of input files and settings
-                self.set_input_paths(self.case_model)
+                self.set_input_paths()
 
                 # Build circuit model
                 for s in self.software:
                     if s == 'PSPICE': self.buildPSPICE()
 
         ### Case of a conductor model
-        elif case_model == 'conductor':
+        elif self.case_model == 'conductor':
             # Initialize
             self.conductor_data: DataModelConductor = DataModelConductor()
             self.path_settings = None
@@ -142,17 +140,17 @@ class BuilderModel:
                     print('output_path: {}'.format(self.output_path))
 
                 # Load model data from the input .yaml file
-                self.loadModelData(self.case_model)
+                self.loadModelData()
 
                 # Set paths of input files and settings
-                self.set_input_paths(self.case_model)
+                self.set_input_paths()
 
                 # Build model
                 for s in self.software:
-                    if s == 'LEDET':    self.buildLEDET(case_model=self.case_model)
+                    if s == 'LEDET':    self.buildLEDET()
                     if s == 'PyBBQ':    self.buildPyBBQ()
 
-    def set_input_paths(self, case_model: str):
+    def set_input_paths(self):
         """
             Sets input paths and displays related information
         """
@@ -165,7 +163,8 @@ class BuilderModel:
         self.path_cadata = None
         self.path_iron = None
 
-        if case_model == 'magnet':
+
+        if self.case_model == 'magnet':
             # Set a few paths relative to the "anchor" path
             # If input paths are not defined, their value remains to their default None
             # The construct Path(x / y).resolve() allows defining relative paths in the .yaml input file
@@ -177,14 +176,14 @@ class BuilderModel:
                 self.path_cadata = Path(self.path_magnet / self.model_data.Sources.conductor_fromROXIE).resolve()
             if self.model_data.Sources.iron_fromROXIE:
                 self.path_iron = Path(self.path_magnet / self.model_data.Sources.iron_fromROXIE).resolve()
-        elif case_model in 'circuit':
+        elif self.case_model in 'circuit':
             pass  # no paths to assign
-        elif case_model == 'conductor':
+        elif self.case_model == 'conductor':
             if self.conductor_data.Sources.magnetic_field_fromROXIE:
                 self.path_map2d = Path(
                     self.path_magnet / self.conductor_data.Sources.magnetic_field_fromROXIE).resolve()
         else:
-            raise Exception('case_model ({}) no supported'.format(case_model))
+            raise Exception('case_model ({}) no supported'.format(self.case_model))
 
         # Set settings path
         # Find user name
@@ -201,13 +200,10 @@ class BuilderModel:
         else:
             self.path_settings = Path(os.getcwd() / self.relative_path_settings).resolve()
 
-        if Path.exists(Path.joinpath(self.path_settings, f"settings.{user_name}.yaml")):
-            with open(Path.joinpath(self.path_settings, f"settings.{user_name}.yaml"), 'r') as stream:
-                self.settings_dict = yaml.safe_load(stream)
-        else:
-            with open(Path.joinpath(Path(os.getcwd()).parent, "settings.SYSTEM.yaml"), 'r') as stream:
-                self.settings_dict = yaml.safe_load(stream)
-            # raise Exception('Cannot find paths without settings file')
+        settings_file = Path.joinpath(self.path_settings, f"settings.{user_name}.yaml")
+        if not Path.exists(settings_file):
+            settings_file = Path.joinpath(Path(os.getcwd()).parent, "settings.SYSTEM.yaml")
+        self.settings_dict = yaml_to_dict(settings_file)
 
         # Display defined paths
         if self.verbose:
@@ -217,8 +213,9 @@ class BuilderModel:
             print('path_iron:     {}'.format(self.path_iron))
             print('path_map2d:    {}'.format(self.path_map2d))
             print('path_settings: {}'.format(self.path_settings))
+            print(f'path to settings file: {settings_file}')
 
-    def loadModelData(self, case_model: str):
+    def loadModelData(self):
         """
             Loads model data from yaml file to model data object
         """
@@ -228,20 +225,18 @@ class BuilderModel:
         if not self.file_model_data:
             raise Exception('No .yaml path provided.')
 
-        if case_model == 'magnet':
+        if self.case_model == 'magnet':
             # Load yaml keys into DataModelMagnet dataclass
-            with open(self.file_model_data, "r") as stream:
-                dictionary_yaml = yaml.safe_load(stream)
-                self.model_data = DataModelMagnet(**dictionary_yaml)
-        elif case_model == 'circuit':
+            self.model_data = yaml_to_data_class(self.file_model_data, DataModelMagnet)
+        elif self.case_model == 'circuit':
             Parser = ParserPSPICE(self.circuit_data)
             Parser.readFromYaml(self.file_model_data)
             self.circuit_data = Parser.circuit_data
-        elif case_model == 'conductor':
+        elif self.case_model == 'conductor':
             # Load yaml keys into DataModelConductor dataclass
-            with open(self.file_model_data, "r") as stream:
-                dictionary_yaml = yaml.safe_load(stream)
-                self.conductor_data = DataModelConductor(**dictionary_yaml)
+            self.conductor_data = yaml_to_data_class(self.file_model_data, DataModelConductor)
+        else:
+            raise ValueError('results: case_model must be one of {}.'.format(['magnet', 'circuit', 'conductor']))
 
     def loadRoxieData(self):
         """
@@ -273,21 +268,21 @@ class BuilderModel:
         """
             Building a FiQuS model
         """
-        builder_FiQuS = BuilderFiQuS(input_model_data=self.model_data, input_roxie_data=self.roxie_data,
+
+        builder_FiQuS = BuilderFiQuS(model_data=self.model_data, roxie_data=self.roxie_data,
                                      flag_build=self.flag_build, flag_plot_all=self.flag_plot_all, verbose=self.verbose)
 
         # Write output files
-        parser_FiQuS = ParserFiQuS(builder_FiQuS)
-        parser_FiQuS.writeFiQuS2yaml(full_path_file_name=os.path.join(self.output_path,
-                                     self.model_data.GeneralParameters.magnet_name), verbose=self.verbose)
+        self.parser_FiQuS = ParserFiQuS(builder_FiQuS, verbose=self.verbose)
+        self.parser_FiQuS.writeFiQuS2yaml(output_path=self.output_path, append_str_to_magnet_name='_FiQuS')
 
-    def buildLEDET(self, case_model: str = 'magnet'):
+    def buildLEDET(self):
         """
             Building a LEDET model
             case_model is a string defining the type of model to build (magnet or conductor)
         """
 
-        if case_model == 'magnet':
+        if self.case_model == 'magnet':
             magnet_name = self.model_data.GeneralParameters.magnet_name
             nameFileSMIC = os.path.join(self.output_path, magnet_name + '_selfMutualInductanceMatrix.csv')  # full path of the .csv file with self-mutual inductances to write
 
@@ -350,7 +345,7 @@ class BuilderModel:
             builder_ledet = BuilderLEDET(path_magnet=self.path_magnet, input_model_data=self.model_data,
                                          input_roxie_data=self.roxie_data, input_map2d=self.map2d_file_modified,
                                          smic_write_path=nameFileSMIC, flag_build=self.flag_build,
-                                         flag_plot_all=self.flag_plot_all, verbose=self.verbose, case_model=case_model)
+                                         flag_plot_all=self.flag_plot_all, verbose=self.verbose, case_model=self.case_model)
 
             # Copy or modify+copy magnet-name_E....map2d files
             number_input_files = len([entry for entry in os.listdir(self.path_magnet) if
@@ -390,7 +385,7 @@ class BuilderModel:
                 parser_ledet.write2json(full_path_file_name=nameFileLedetJson, verbose=self.verbose,
                                         SkipConsistencyCheck=True)
 
-        elif case_model == 'conductor':
+        elif self.case_model == 'conductor':
             conductor_name = self.conductor_data.GeneralParameters.conductor_name
 
             # Copy the ROXIE map2d file, if defined in the input file
@@ -423,7 +418,7 @@ class BuilderModel:
                                          input_map2d=self.map2d_file_modified,
                                          flag_build=self.flag_build, flag_plot_all=self.flag_plot_all,
                                          verbose=self.verbose,
-                                         case_model=case_model)
+                                         case_model=self.case_model)
 
             # Write output excel file
             parser_ledet = ParserLEDET(builder_ledet)
@@ -443,20 +438,20 @@ class BuilderModel:
                                         SkipConsistencyCheck=True)
 
         else:
-            raise Exception('Case model {} is not supported when building a LEDET model.'.format(case_model))
+            raise Exception('Case model {} is not supported when building a LEDET model.'.format(self.case_model))
 
     def buildProteCCT(self):
         """
             Building a ProteCCT model
         """
+
         magnet_name = self.model_data.GeneralParameters.magnet_name
         builder_protecct = BuilderProteCCT(input_model_data=self.model_data, flag_build=self.flag_build,
                                            verbose=self.verbose)
 
         # Write output excel file
         parser_protecct = ParserProteCCT(builder_protecct)
-        nameFileProteCCT = os.path.join(self.output_path,
-                                        magnet_name + '.xlsx')  # full path of the ProteCCT input file to write
+        nameFileProteCCT = os.path.join(self.output_path, f'{magnet_name}_ProteCCT.xlsx')  # full path of the ProteCCT input file to write
         parser_protecct.writeProtecct2Excel(full_path_file_name=nameFileProteCCT, verbose=self.verbose,
                                             SkipConsistencyCheck=True)
 
@@ -464,13 +459,12 @@ class BuilderModel:
         """
             Build a PSPICE circuit netlist model
         """
-        circuit_data = self.circuit_data
-        circuit_name = self.circuit_data.GeneralParameters.circuit_name
+
 
         # Write output .cir file
-        parser_pspice = ParserPSPICE(circuit_data=circuit_data, path_input=self.path_magnet,
+        parser_pspice = ParserPSPICE(circuit_data=self.circuit_data, path_input=self.path_magnet,
                                      output_path=self.output_path)
-        nameFilePSPICE = os.path.join(self.output_path, circuit_name + '.cir')  # full path of the PSPICE netlist to write
+        nameFilePSPICE = os.path.join(self.output_path, self.circuit_data.GeneralParameters.circuit_name + '.cir')  # full path of the PSPICE netlist to write
         parser_pspice.write2pspice(full_path_file_name=nameFilePSPICE, verbose=self.verbose)
 
         # Copy additional files
@@ -480,6 +474,7 @@ class BuilderModel:
         """
             Building a PyBBQ model
         """
+
         conductor_name = self.conductor_data.GeneralParameters.conductor_name
         builder_PyBBQ = BuilderPyBBQ(input_model_data=self.conductor_data, flag_build=self.flag_build,
                                      verbose=self.verbose)
@@ -493,6 +488,7 @@ class BuilderModel:
         """
             Building a SIGMA model
         """
+
         BuilderSIGMA(input_model_data=self.model_data, input_roxie_data=self.roxie_data,
                      settings_dict=self.settings_dict, output_path=self.output_path,
                      flag_build=self.flag_build, verbose=self.verbose)
