@@ -131,6 +131,41 @@ class PostProcess(BaseModel):
     type: Literal['PostProcess']
 
 
+class RunCustomPyFunction(BaseModel):
+    """
+        Level 2: Analysis step to run a custom Python function
+    """
+    type: Literal['RunCustomPyFunction']
+    flag_enable: bool = None
+    function_name: str = None
+    function_arguments: Dict = {}
+    path_module: str = None  # optional
+
+
+class RunViewer(BaseModel):
+    """
+        Level 2: Analysis step to make a steam_sdk.viewers.Viewer.Viewer() object and run its analysis
+    """
+    type: Literal['RunViewer']
+    viewer_name: str = None
+    file_name_transients: str = None
+    list_events: List[int] = []
+    flag_analyze: bool = None
+    flag_display: bool = None
+    flag_save_figures: bool = None
+    verbose: bool = None
+
+
+class CalculateMetrics(BaseModel):
+    """
+        Level 2: Analysis step to calculate metrics (usually to compare two or more measured and/or simulated signals)
+    """
+    type: Literal['CalculateMetrics']
+    viewer_name: str = None
+    metrics_to_calculate: List[str] = []
+    variables_to_analyze: List[List[str]] = [[]]
+
+
 # class AnalysisStep(BaseModel):
 #     """
 #         Level 1: Class for information on the analysis step
@@ -157,5 +192,7 @@ class DataAnalysis(BaseModel):
     GeneralParameters: General = General()
     PermanentSettings: DataSettings = DataSettings()
     WorkingFolders: WorkingFolders = WorkingFolders()
-    AnalysisStepDefinition: Dict[str, Union[MakeModel, ModifyModel, ModifyModelMultipleVariables, SetUpFolder, AddAuxiliaryFile, CopyFile, RunSimulation, PostProcess]] = {}
+    AnalysisStepDefinition: Dict[str, Union[MakeModel, ModifyModel, ModifyModelMultipleVariables, SetUpFolder,
+                                            AddAuxiliaryFile, CopyFile, RunSimulation, PostProcess, RunCustomPyFunction,
+                                            RunViewer, CalculateMetrics]] = {}
     AnalysisStepSequence: List[str] = []  # Here the analysis steps are defined, in execution order. Names must be defined in AnalysisStepDefinition. Repetitions ARE allowed.
