@@ -3,12 +3,10 @@ from typing import (Dict, List, Union, Literal)
 from steam_sdk.data.DataRoxieParser import RoxieData
 from steam_sdk.data.DataModelMagnet import RunFiQuS
 from steam_sdk.data.DataModelMagnet import MeshCCT
-from steam_sdk.data.DataModelMagnet import SolveCCT
-from steam_sdk.data.DataModelMagnet import FQPL_g
 from steam_sdk.data.DataModelMagnet import Air_g
-from steam_sdk.data.DataModelMagnet import OptionMultipole
-from steam_sdk.data.DataModelMagnet import MeshMultipole
-from steam_sdk.data.DataModelMagnet import PostProcMultipole
+from steam_sdk.data.DataModelMagnet import MultipoleGeometry
+from steam_sdk.data.DataModelMagnet import MultipoleMesh
+from steam_sdk.data.DataModelMagnet import MultipolePostProc
 
 
 class MultipoleRibbon(BaseModel):
@@ -58,7 +56,7 @@ class MultipoleSettings(BaseModel):
     Model_Data_GS: MultipoleModelDataSetting = MultipoleModelDataSetting()
 
 
-class MultipoleGeometry(BaseModel):
+class MultipoleRoxieGeometry(BaseModel):
     """
         Class for FiQuS multipole Roxie data (.geom)
     """
@@ -69,8 +67,6 @@ class MultipoleGeometry(BaseModel):
 class MultipoleSolve(BaseModel):
     I_initial: List[float] = None
     pro_template: str = None  # file name of .pro template file
-    getdp_exe_path: str = None  # absolute path to GetDP executable
-    BH_curves_path: str = None  # absolute path to iron BH curves pro file
 ############################
 
 
@@ -102,6 +98,21 @@ class CCTFormerG(BaseModel):  # Geometry related formers _inputs
     z_maxs: List[float] = None  # extend of former in positive z direction
 
 
+class FQPL_g(BaseModel):
+    """
+        Level 2: Class for FiQuS CCT
+    """
+    names: List[str] = None  # name to use in gmsh and getdp
+    fndpls: List[int] = None  # fqpl number of divisions per length
+    fwws: List[float] = None  # fqpl wire widths (assuming rectangular) for theta = 0 this is x dimension
+    fwhs: List[float] = None  # fqpl wire heights (assuming rectangular) for theta = 0 this is y dimension
+    r_ins: List[float] = None  # radiuses for inner diameter for fqpl (radial (or x direction for theta=0) for placing the fqpl
+    r_bs: List[float] = None  # radiuses for bending the fqpl by 180 degrees
+    n_sbs: List[int] = None  # number of 'bending segmetns' for the 180 degrees turn
+    thetas: List[float] = None  # rotation in deg from x+ axis towards y+ axis about z axis.
+    z_starts: List[str] = None  # which air boundary to start at. These is string with either: z_min or z_max key from the Air region.
+    z_ends: List[float] = None  # z coordinate of loop end
+
 class CCTGeometry(BaseModel):
     """
         Level 2: Class for FiQuS CCT for FiQuS input
@@ -111,6 +122,49 @@ class CCTGeometry(BaseModel):
     formers: CCTFormerG = CCTFormerG()
     air: Air_g = Air_g()
 
+class Winding_s(BaseModel):  # Solution time used windings _inputs (materials and BC)
+    """
+        Level 2: Class for FiQuS CCT
+    """
+    currents: List[float] = None  # current in the wire
+    sigmas: List[float] = None  # electrical conductivity
+    mu_rs: List[float] = None  # relative permeability
+
+class FQPL_s(BaseModel):  # Solution time used windings _inputs (materials and BC)
+    """
+        Level 2: Class for FiQuS CCT
+    """
+    currents: List[float] = None  # current in the wire
+    sigmas: List[float] = None  # electrical conductivity
+    mu_rs: List[float] = None  # relative permeability
+
+class Former_s(BaseModel):  # Solution time used formers _inputs (materials and BC)
+    """
+        Level 2: Class for FiQuS CCT
+    """
+    sigmas: List[float] = None  # electrical conductivity
+    mu_rs: List[float] = None  # relative permeability
+
+class Air_s(BaseModel):  # Solution time used air _inputs (materials and BC)
+    """
+        Level 2: Class for FiQuS CCT
+    """
+    sigma: float = None  # electrical conductivity
+    mu_r: float = None  # relative permeability
+
+class SolveCCT(BaseModel):
+    """
+        Level 2: Class for FiQuS CCT
+    """
+    windings: Winding_s = Winding_s()  # windings solution time _inputs
+    formers: Former_s = Former_s()  # former solution time _inputs
+    fqpls: FQPL_s = FQPL_s() # fqpls solution time _inputs
+    air: Air_s = Air_s()  # air solution time _inputs
+    pro_template: str = None  # file name of .pro template file
+    getdp_exe_path: str = None  # absolute path to GetDP executable
+    variables: List[str] = None  # Name of variable to post-process by GetDP, like B for magnetic flux density
+    volumes: List[str] = None  # Name of volume to post-process by GetDP, line Winding_1
+    file_exts: List[str] = None  # Name of file extensions to post-process by GetDP, like .pos
 
 class CCTPostproc(BaseModel):
     """
@@ -143,10 +197,10 @@ class MPDM(BaseModel):
         Class for FiQuS multipole
     """
     type: Literal['multipole']
-    options: OptionMultipole = OptionMultipole()
-    mesh: MeshMultipole = MeshMultipole()
+    geometry: MultipoleGeometry = MultipoleGeometry()
+    mesh: MultipoleMesh = MultipoleMesh()
     solve: MultipoleSolve = MultipoleSolve()
-    post_proc: PostProcMultipole = PostProcMultipole()
+    postproc: MultipolePostProc = MultipolePostProc()
 
 
 class General(BaseModel):
