@@ -198,6 +198,26 @@ class CLIQ(BaseModel):
     I0: float = None                       # I_cliq_0
 
 
+class FQPLs(BaseModel):  # Geometry related fqpls _inputs
+    """
+        Level 2: Class for the FQPLs parameters for protection
+    """
+    enabled: bool = None  # are fqpls enabled
+    names: List[str] = None  # name to use in gmsh and getdp
+    fndpls: List[int] = None  # fqpl number of divisions per length
+    fwws: List[float] = None  # fqpl wire widths (assuming rectangular) for theta = 0 this is x dimension
+    fwhs: List[float] = None  # fqpl wire heights (assuming rectangular) for theta = 0 this is y dimension
+    r_ins: List[float] = None  # radiuses for inner diameter for fqpl (radial (or x direction for theta=0) for placing the fqpl
+    r_bs: List[float] = None  # radiuses for bending the fqpl by 180 degrees
+    n_sbs: List[int] = None  # number of 'bending segmetns' for the 180 degrees turn
+    thetas: List[float] = None  # rotation in deg from x+ axis towards y+ axis about z axis.
+    z_starts: List[str] = None  # which air boundary to start at. These is string with either: z_min or z_max key from the Air region.
+    z_ends: List[float] = None  # z coordinate of loop end
+    currents: List[float] = None  # current in the wire
+    sigmas: List[float] = None  # electrical conductivity
+    mu_rs: List[float] = None  # relative permeability
+
+
 class QuenchProtection(BaseModel):
     """
         Level 1: Class for quench protection
@@ -205,6 +225,7 @@ class QuenchProtection(BaseModel):
     Energy_Extraction: EnergyExtraction = EnergyExtraction()
     Quench_Heaters: QuenchHeater = QuenchHeater()
     CLIQ: CLIQ = CLIQ()
+    FQPLs: FQPLs = FQPLs()
 
 
 ############################
@@ -272,20 +293,18 @@ class Threshold(BaseModel):
     DistMax: float = None
 
 
-class OptionMultipole(BaseModel):
+class MultipoleGeometry(BaseModel):
     """
         Level 2: Class for FiQuS Multipole
     """
     with_iron_yoke: bool = None
-    compare_to_ROXIE: bool = None
-    default_mesh: bool = None
-    plot_all: bool = None
 
 
-class MeshMultipole(BaseModel):
+class MultipoleMesh(BaseModel):
     """
         Level 2: Class for FiQuS Multipole
     """
+    default_mesh: bool = None
     mesh_iron: Threshold = Threshold()
     mesh_coil: Threshold = Threshold()
     MeshSizeMin: float = None  # sets gmsh Mesh.MeshSizeMin
@@ -297,19 +316,19 @@ class MeshMultipole(BaseModel):
     Optimize: int = None  # sets gmsh Mesh.Optimize
 
 
-class SolveMultipole(BaseModel):
+class MultipoleSolve(BaseModel):
     """
         Level 2: Class for FiQuS Multipole
     """
     pro_template: str = None  # file name of .pro template file
-    getdp_exe_path: str = None  # absolute path to GetDP executable
-    BH_curves_path: str = None  # absolute path to iron BH curves pro file
 
 
-class PostProcMultipole(BaseModel):
+class MultipolePostProc(BaseModel):
     """
         Level 2: Class for FiQuS Multipole
     """
+    compare_to_ROXIE: str = None
+    plot_all: bool = None
     variables: List[str] = None  # Name of variables to post-process, like "b" for magnetic flux density
     volumes: List[str] = None  # Name of domains to post-process, like "powered"
     file_exts: List[str] = None  # Name of file extensions to output to, like "pos"
@@ -320,10 +339,10 @@ class FiQuSMultipole(BaseModel):
     """
         Level 2: Class for FiQuS Multipole
     """
-    options: OptionMultipole = OptionMultipole()
-    mesh: MeshMultipole = MeshMultipole()
-    solve: SolveMultipole = SolveMultipole()
-    post_proc: PostProcMultipole = PostProcMultipole()
+    geometry: MultipoleGeometry = MultipoleGeometry()
+    mesh: MultipoleMesh = MultipoleMesh()
+    solve: MultipoleSolve = MultipoleSolve()
+    postproc: MultipolePostProc = MultipolePostProc()
 
 
 # CCT
@@ -353,31 +372,6 @@ class Winding_s(BaseModel):  # Solution time used windings _inputs (materials an
     mu_rs: List[float] = None  # relative permeability
 
 
-class FQPL_g(BaseModel):  # Geometry related fqpls _inputs
-    """
-        Level 2: Class for FiQuS CCT
-    """
-    names: List[str] = []  # name to use in gmsh and getdp
-    fndpls: List[int] = None  # fqpl number of divisions per length
-    fwws: List[float] = None  # fqpl wire widths (assuming rectangular) for theta = 0 this is x dimension
-    fwhs: List[float] = None  # fqpl wire heights (assuming rectangular) for theta = 0 this is y dimension
-    r_ins: List[float] = None  # radiuses for inner diameter for fqpl (radial (or x direction for theta=0) for placing the fqpl
-    r_bs: List[float] = None  # radiuses for bending the fqpl by 180 degrees
-    n_sbs: List[int] = None  # number of 'bending segmetns' for the 180 degrees turn
-    thetas: List[float] = None  # rotation in deg from x+ axis towards y+ axis about z axis.
-    z_starts: List[str] = None  # which air boundary to start at. These is string with either: z_min or z_max key from the Air region.
-    z_ends: List[float] = None  # z coordinate of loop end
-
-
-class FQPL_s(BaseModel):  # Solution time used fqpls _inputs (materials and BC)
-    """
-        Level 2: Class for FiQuS CCT
-    """
-    currents: List[float] = []  # current in the wire
-    sigmas: List[float] = []  # electrical conductivity
-    mu_rs: List[float] = []  # relative permeability
-
-
 class Former_g(BaseModel):  # Geometry related formers _inputs
     """
         Level 2: Class for FiQuS CCT Options
@@ -420,7 +414,6 @@ class GeometryCCT(BaseModel):
         Level 2: Class for FiQuS CCT
     """
     windings: Winding_g = Winding_g()
-    fqpls: FQPL_g = FQPL_g()
     formers: Former_g = Former_g()
     air: Air_g = Air_g()
 
@@ -441,7 +434,6 @@ class SolveCCT(BaseModel):
         Level 2: Class for FiQuS CCT
     """
     windings: Winding_s = Winding_s()  # windings solution time _inputs
-    fqpls: FQPL_s = FQPL_s()  # fqpls solution time _inputs
     formers: Former_s = Former_s()  # former solution time _inputs
     air: Air_s = Air_s()  # air solution time _inputs
     pro_template: str = None  # file name of .pro template file
@@ -551,6 +543,7 @@ class FieldMapFilesLEDET(BaseModel):
     flagPlotMTF: int = None
     fieldMapNumber: int = None
     flag_modify_map2d_ribbon_cable: int = None
+    flag_calculateMagneticField: int = None
 
 
 class InputGenerationOptionsLEDET(BaseModel):
