@@ -1,4 +1,7 @@
+import os
+import numpy as np
 import pandas as pd
+from pathlib import Path
 from steam_sdk.parsers.CSD_Reader import CSD_read
 
 def get_signals_from_csd(full_name_file: str, list_signals):
@@ -13,12 +16,9 @@ def get_signals_from_csd(full_name_file: str, list_signals):
     csd=CSD_read(full_name_file).data_dict
     csd['time'] = csd['Time']
     del csd['Time']
-    # print(csd)
+    print(csd)
     all_signals_df = pd.DataFrame.from_dict(csd) # read file into a dataframe
-    # print(all_signals_df)
-    #all_signals_df.columns = all_signals_df.columns.str.replace(' ', '')  # eliminate whitespaces from the column names
-    #list_signals = [x.replace(' ', '') for x in list_signals]             # eliminate whitespaces from the signal names as well
+    print(all_signals_df)
+    all_signals_df.columns = all_signals_df.columns.str.replace(' ', '')  # eliminate whitespaces from the column names
+    list_signals = [x.replace(' ', '') for x in list_signals]             # eliminate whitespaces from the signal names
     return all_signals_df[list_signals]
-
-
-#get_signals_from_csd('C:/cernbox/steam-models-dev/analyses/example_circuit_analysis/output/PSPICE/RCS.csd',['I(x_R_EE.v_virtual_in)','V(x_R_EE.304)','I(R_WARM_1)','I(x_Crowbar.R_crowbar)','I(x_EC.v_virtual_in)'])
\ No newline at end of file
