@@ -22,7 +22,7 @@ class ParserLEDET:
         Class with methods to read/write LEDET information from/to other programs
     """
 
-    def __init__(self, builder_ledet: BuilderLEDET = BuilderLEDET(flag_build=False)):
+    def __init__(self, builder_ledet: BuilderLEDET):
         """
             Initialization using a BuilderLEDET object containing LEDET parameter structure
         """
@@ -948,6 +948,7 @@ def _getOptionalVariables_input():
 def _getOptionalVariables_output():
     optional_variables_output_sheet = [
         'fieldMapNumber',
+        'flag_calculateMagneticField',
         ]
     return optional_variables_output_sheet
 
